const helpers = require('../../helpers/helpers')
const uuid = require('uuid')
const authsomeHelper = require('../../helpers/authsome')
const collectionHelper = require('../../helpers/Collection')
const userHelper = require('../../helpers/User')
const mailService = require('pubsweet-component-mail-service')

module.exports = models => async (req, res) => {
  const { recommendation, comments, recommendationType } = req.body

  if (!helpers.checkForUndefinedParams(recommendationType))
    return res.status(400).json({ error: 'Recommendation type is required.' })

  const reqUser = await models.User.find(req.user)
  const { collectionId, fragmentId } = req.params

  let collection, fragment

  try {
    collection = await models.Collection.find(collectionId)
    if (!collection.fragments.includes(fragmentId))
      return res.status(400).json({
        error: `Collection and fragment do not match.`,
      })

    fragment = await models.Fragment.find(fragmentId)
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'Item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
  const authsome = authsomeHelper.getAuthsome(models)
  const target = {
    collection,
    path: req.route.path,
  }

  const canPost = await authsome.can(req.user, 'POST', target)
  if (!canPost)
    return res.status(403).json({
      error: 'Unauthorized.',
    })
  fragment.recommendations = fragment.recommendations || []
  const newRecommendation = {
    id: uuid.v4(),
    userId: reqUser.id,
    createdOn: Date.now(),
    updatedOn: Date.now(),
    recommendationType,
  }

  newRecommendation.recommendation = recommendation || undefined
  newRecommendation.comments = comments || undefined

  if (recommendationType === 'editorRecommendation') {
    await collectionHelper.updateStatusByRecommendation(
      collection,
      recommendation,
    )
    const {
      title,
      submittingAuthor,
      id,
    } = await collectionHelper.getFragmentAndAuthorData({
      UserModel: models.User,
      FragmentModel: models.Fragment,
      collection,
    })
    const authorName = `${submittingAuthor.firstName} ${
      submittingAuthor.lastName
    }`
    const baseUrl = helpers.getBaseUrl(req)
    await userHelper.setupAgreedReviewersEmailData({
      baseUrl,
      UserModel: models.User,
      FragmentModel: models.Fragment,
      collection,
      mailService,
      fragment: { title, authorName },
      recommendation,
    })
    await userHelper.setupEiCRecommendationEmailData({
      baseUrl,
      UserModel: models.User,
      collection,
      mailService,
      recommendation,
      fragment: { title, id, authorName },
      comments: newRecommendation.comments,
    })
    await userHelper.setupAuthorRequestToRevisionEmailData({
      baseUrl,
      UserModel: models.User,
      collection,
      mailService,
      recommendation,
      fragment: { title, id, submittingAuthor },
      comments: newRecommendation.comments,
    })
  }

  fragment.recommendations.push(newRecommendation)
  await fragment.save()
  return res.status(200).json(newRecommendation)
}
